//
//  LocationDelegate.m
//  Near TEECOM
//
//  Created by Alex Serriere on 12/14/11.
//  Copyright (c) 2011 TEECOM. All rights reserved.
//

#import "LocationDelegate.h"
#import "ViewController.h"
#import "MyMapAnnotation.h"

#define oaklandOfficeLatitude 37.80398592212445
#define oaklandOfficeLongitude -122.27208137512207
#define metersToMiles 0.000621371192

@interface LocationDelegate(private)
-(void)zoomToFitMapAnnotations:(MKMapView*)theMapView;
@end

@implementation LocationDelegate

@synthesize locationManager;


- (void)locationManager:(CLLocationManager *)manager
    didUpdateToLocation:(CLLocation *)newLocation
           fromLocation:(CLLocation *)oldLocation
{

        NSLog(@"latitude %+.6f, longitude %+.6f\n",
              newLocation.coordinate.latitude,
              newLocation.coordinate.longitude);
        
        AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
        
        ViewController *theView = (ViewController *) appDelegate.window.rootViewController;
    
    CLLocationCoordinate2D teecomCoordinates = CLLocationCoordinate2DMake(oaklandOfficeLatitude, oaklandOfficeLongitude);
    
    CLLocation *headquarters = [[CLLocation alloc] initWithCoordinate:teecomCoordinates altitude:0 horizontalAccuracy:1 verticalAccuracy:1 timestamp:[NSDate date]];
        
        
        [theView.currentLongitude setText:[NSString stringWithFormat:@"%+.6f,%+.6f",newLocation.coordinate.longitude,newLocation.coordinate.latitude]];
    [theView.distanceToTEECOM setText:[NSString stringWithFormat:@"%f miles",[newLocation distanceFromLocation:headquarters]*metersToMiles]]; 
         
    MyMapAnnotation *teecomHQ = [[MyMapAnnotation alloc] init];
    
    [teecomHQ setLatitude:[NSNumber numberWithDouble:oaklandOfficeLatitude]];
    [teecomHQ setLongitude:[NSNumber numberWithDouble:oaklandOfficeLongitude]];
    
    if ([theView.mapView.annotations count] < 2) {
        [theView.mapView addAnnotation:teecomHQ];
    }
    
    [self zoomToFitMapAnnotations:theView.mapView];
    
    NSLog(@"Annoations shown:%i",[theView.mapView.annotations count]);
        /*AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
         
         UITabBarController *tabBarController = (UITabBarController *)  appDelegate.window.rootViewController;
         
         UINavigationController *navigationController = [[tabBarController viewControllers] objectAtIndex:1];
         
         MapViewController *mapViewController = [[navigationController viewControllers] objectAtIndex:0];
         
         [mapViewController.mapView showsUserLocation];*/
        
        
}

- (void)locationManager:(CLLocationManager *)manager didEnterRegion:(CLRegion *)region
{
    // Just log that this happened for now
    NSLog(@"Entered region...");
    
    UILocalNotification *localNotif = [[UILocalNotification alloc] init];
    if (localNotif) {
        
        NSLog(@"latitude %+.6f, longitude %+.6f\n",
              manager.location.coordinate.latitude,
              manager.location.coordinate.longitude);

        CLLocation *regionCenter = [[CLLocation alloc] initWithCoordinate:[region center] altitude:0 horizontalAccuracy:1 verticalAccuracy:1 timestamp:[NSDate date]];
        
        float distanceToRegionCenter = [manager.location distanceFromLocation:regionCenter]*metersToMiles;
        
        localNotif.alertBody = [NSString stringWithFormat:
                                NSLocalizedString(@"You're %.2f miles from %@", nil), distanceToRegionCenter, region.identifier];
        localNotif.alertAction = NSLocalizedString(@"Read Message", nil);
        localNotif.soundName = @"alarmsound.caf";
        localNotif.applicationIconBadgeNumber = 0;
        
        UIApplication *theApplication = [UIApplication sharedApplication];
        
        [theApplication presentLocalNotificationNow:localNotif];
    }
    
}

- (void)startMonitoringForRegion
{
    // Create the location manager if this object does not
    // already have one.
    if (nil == locationManager)
        locationManager = [[CLLocationManager alloc] init];
    
    CLLocationCoordinate2D regionCenter = CLLocationCoordinate2DMake(oaklandOfficeLatitude, oaklandOfficeLongitude);
    
    CLRegion *oaklandOfficeRegion = [[CLRegion alloc] initCircularRegionWithCenter:regionCenter radius:500 identifier:@"Headquarters"];
        
    locationManager.delegate = self;
    [locationManager startMonitoringForRegion:oaklandOfficeRegion];
}

- (void)startMonitoringForSignificantChanges
{
    if (locationManager == Nil)
        locationManager = [[CLLocationManager alloc] init];
    
    locationManager.delegate = self;
    [locationManager startMonitoringSignificantLocationChanges];
    
}

- (void)startMonitoringForStandardChanges
{
    if (locationManager == Nil)
        locationManager = [[CLLocationManager alloc] init];
    
    locationManager.delegate = self;
    [locationManager startUpdatingLocation];
}

-(void)zoomToFitMapAnnotations:(MKMapView*)theMapView
{
    if([[theMapView annotations] count] == 0)
        return;
    
    CLLocationCoordinate2D topLeftCoord;
    topLeftCoord.latitude = -90;
    topLeftCoord.longitude = 180;
    
    CLLocationCoordinate2D bottomRightCoord;
    bottomRightCoord.latitude = 90;
    bottomRightCoord.longitude = -180;
    
    for(MyMapAnnotation *annotation in theMapView.annotations)
    {
        topLeftCoord.longitude = fmin(topLeftCoord.longitude, annotation.coordinate.longitude);
        topLeftCoord.latitude = fmax(topLeftCoord.latitude, annotation.coordinate.latitude);
        
        bottomRightCoord.longitude = fmax(bottomRightCoord.longitude, annotation.coordinate.longitude);
        bottomRightCoord.latitude = fmin(bottomRightCoord.latitude, annotation.coordinate.latitude);
    }
    
    MKCoordinateRegion region;
    region.center.latitude = topLeftCoord.latitude - (topLeftCoord.latitude - bottomRightCoord.latitude) * 0.5;
    region.center.longitude = topLeftCoord.longitude + (bottomRightCoord.longitude - topLeftCoord.longitude) * 0.5;
    region.span.latitudeDelta = fabs(topLeftCoord.latitude - bottomRightCoord.latitude) * 1.1; // Add a little extra space on the sides
    region.span.longitudeDelta = fabs(bottomRightCoord.longitude - topLeftCoord.longitude) * 1.1; // Add a little extra space on the sides
    
    region = [theMapView regionThatFits:region];
    [theMapView setRegion:region animated:YES];
}

@end
